#!/usr/bin/python3

"""
Output methods that use dot syntax to represent dependency information.
"""

import re
import sys

import devpipeline_core.resolve


def _dotify(string):
    """This function swaps '-' for '_'."""
    return re.sub("-", lambda m: "_", string)


def _do_dot(targets, components, layer_fn):
    def _handle_layer_dependencies(resolved_dependencies, indent, attributes):
        for attribute in attributes:
            print(indent + attribute)
        for component in resolved_dependencies:
            stripped_name = _dotify(component)
            print(indent + stripped_name)
            component_dependencies = components.get(
                component).get_list("depends")
            for dep in component_dependencies:
                print("{}{} -> {}".format(indent, stripped_name, _dotify(dep)))

    print("digraph dependencies {")
    try:
        devpipeline_core.resolve.process_dependencies(
            targets, components, lambda rd: layer_fn(
                rd, lambda rd, indent: _handle_layer_dependencies(rd, indent,
                                                                  [])))
    except devpipeline_core.resolve.CircularDependencyException as cde:
        layer_fn(
            cde.components,
            lambda rd, indent: _handle_layer_dependencies(rd, indent, [
                'edge [color="red"]',
                'node [color="red"]'
            ]))
    print("}")


def _print_layers(targets, components):
    """
    Print dependency information, grouping components based on their position
    in the dependency graph.  Components with no dependnecies will be in layer
    0, components that only depend on layer 0 will be in layer 1, and so on.

    If there's a circular dependency, those nodes and their dependencies will
    be colored red.

    Arguments
    targets - the targets explicitly requested
    components - full configuration for all components in a project
    """
    layer = 0

    def _add_layer(resolved_dependencies, dep_fn):
        nonlocal layer

        indentation = " " * 4
        print("{}subgraph cluster_{} {{".format(indentation, layer))
        print("{}label=\"Layer {}\"".format(indentation * 2, layer))
        dep_fn(resolved_dependencies, indentation * 2)
        print("{}}}".format(indentation))
        layer += 1

    _do_dot(targets, components, _add_layer)


_LAYERS_TOOL = (
    _print_layers,
    "Print a dot graph that groups components by their position in a layered "
    "architecture.  Components are only permitted to depend on layers with a "
    "lower number.")


def _print_graph(targets, components):
    """
    Print dependency information using a dot directed graph.  The graph will
    contain explicitly requested targets plus any dependencies.

    If there's a circular dependency, those nodes and their dependencies will
    be colored red.

    Arguments
    targets - the targets explicitly requested
    components - full configuration for all components in a project
    """
    indentation = " " * 4
    _do_dot(targets, components, lambda rd, dep_fn: dep_fn(rd, indentation))


_GRAPH_TOOL = (
    _print_graph,
    "Print a dot graph where each component points at its dependnet "
    "components.")


def _print_dot(targets, components):
    """
    Deprecated function; use print_graph.

    Arguments
    targets - the targets explicitly requested
    components - full configuration for all components in a project
    """
    print("Warning: dot option is deprecated.  Use graph instead.",
          file=sys.stderr)
    _print_graph(targets, components)


_DOT_TOOL = (_print_dot, "Deprecated -- use the \"graph\" option instead.")



#!/usr/bin/python3
"""This modules generates a build ordered list of targets."""

import argparse

import devpipeline_core.command
import devpipeline_core.plugin
import devpipeline_core.resolve
import devpipeline_configure.cache


_ORDER_OUTPUTS = devpipeline_core.plugin.query_plugins(
    'devpipeline.build_order.methods')


def _list_methods(targets, components):
    # neither argument is required
    del targets
    del components

    for key in sorted(_ORDER_OUTPUTS):
        print("{} - {}".format(key, _ORDER_OUTPUTS[key][1]))


_MAJOR = 0
_MINOR = 3
_PATCH = 0

_STRING = "{}.{}.{}".format(_MAJOR, _MINOR, _PATCH)


class BuildOrderer(devpipeline_core.command.TargetCommand):

    """This class outputs an ordered list of the packages to satisfy dependencies."""

    def __init__(self, config_fn):
        super().__init__(config_fn=config_fn,
                         prog="dev-pipeline build-order",
                         description="Determinte all dependencies of a set of "
                                     "targets and the order they should be "
                                     "built in.")
        self.add_argument("--method",
                          help="The method used to display build order.",
                          default="list")
        self.add_argument("--list-methods", action='store_true',
                          default=argparse.SUPPRESS,
                          help="List the available methods instead of printing"
                               " dependency information.")
        self.set_version(_STRING)
        self.helper_fn = None

    def setup(self, arguments):
        if "list_methods" in arguments:
            self.helper_fn = _list_methods
        else:
            build_order = _ORDER_OUTPUTS.get(arguments.method)
            if not build_order:
                raise Exception("Invalid method: {}".format(arguments.method))
            self.helper_fn = build_order[0]

    def process(self):
        self.helper_fn(self.targets, self.components)


def main(args=None, config_fn=devpipeline_configure.cache.update_cache):
    # pylint: disable=missing-docstring
    build_orderer = BuildOrderer(config_fn)
    devpipeline_core.command.execute_command(build_orderer, args)


_BUILD_ORDER_COMMAND = (
    main, "Generate dependency information about project components.")

if __name__ == '__main__':
    main()



#!/usr/bin/python3

"""
Root module for the build-order tool.
"""

import devpipeline_core.resolve


# For some reason, setup.py can't find this in the build_order moudle.  Move it
# back when I figure out why.
def _print_list(targets, components):
    build_order = devpipeline_core.resolve.order_dependencies(
        targets, components)
    print(build_order)


_LIST_TOOL = (_print_list, "Print a sequential order of components.")
